# -*- coding: utf-8 -*-
# Create a geo (gmsh input file) file from a contour file
# the contour file contains the (x,y) coordinates of the ordered
# points defining the contour of the domain

# https://albertsk.files.wordpress.com/2012/12/gmsh_tutorial.pdf
# https://bthierry.pages.math.cnrs.fr/tutorial/gmsh/basics_physical_vs_elementary/

import sys
import argparse
import numpy as np
import matplotlib.pyplot as plt

# ------------
def main(argv):
    # Parse command line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("-p",  "--profile", required = True,
                        help = "Filepath to bed profile. ASCII format")
    parser.add_argument("-o",  "--output", required = False,
                        help = "Filename of output .geo file")
    parser.add_argument("-dx", "--resolution", required = False,
                        default=200, help = "Resolution of Mesh")
    parser.add_argument("-sp", "--spline", required = False, action='store_true',
                        help = "Whether or not too spline the points together.")

    args, _ = parser.parse_known_args(argv)

    fp = args.profile
    out_fp  = args.output
    el_size = args.resolution

    # Spline or line
    spline = args.spline

    Contour = np.loadtxt(fp)
    x = Contour[:,0]
    y = Contour[:,1]

    if x[0]==x[-1] and y[0]==y[-1]:
        print('Same first and last points in contour file')
        Npt = len(x)-1
    else:
        Npt = len(x)

    # Open the output file
    geo = open(out_fp, 'w')
    geo.write('// This a a geo file created using the python script Makegeo.py // \n')
    #geo.write('Mesh.Algorithm=5; \n')
    geo.write('// To controle the element size, one can directly modify the lc value in the geo file // \n')
    geo.write('lc = {} ; \n'.format(el_size))

    # write the points coordinates (x,y,0,lc)
    nps=0
    for j in range(0,Npt):
        #if j % 25 == 0 or j == Npt-1:
            #print(j)
        nps=nps+1
        geo.write('Point({}) = '.format(nps)+r'{'+' {:.3f}, {:.3f}, 0.0, lc'.format(x[j],y[j])+r'}'+'; \n')
    nps+=1
    # if spline
    if spline:
        geo.write('Line(1) = {')
        for j in range(0,Npt):
            geo.write('{},'.format(j+1))
        geo.write('1}; \n')

        geo.write('// Volume number for whole domain.\n')
        geo.write('Physical Surface({}) = {{{}}};\n'.format(6,5))
        geo.write('// Top and bottom of the glacier.\n')
        geo.write('Physical Line({}) = {{{}}};\n'.format(7,1))
        geo.write('Physical Line({}) = {{{}}};\n'.format(8,2))
        geo.write('// Left  Wall.\n')
        geo.write('Physical Line({}) = {{{}}};\n'.format(9,3))
        geo.write('// Right Wall.\n')
        geo.write('Physical Line({}) = {{{}}};\n'.format(10,4))

    # else it is lines, as a spline might not work in all case
    else:
        ########################################################################
        # THIS !!!!!DOES NOT!!!!!! WORKS .......... kinda
        ########################################################################
        # nl=0
        # for j in range(0,Npt-1):
        #     nl=nl+1
        #     geo.write('Line({0}) = '.format(nl)+r'{'+'{0},{1}'.format(j+1,j+2)+r'}'+'; \n')
        # #geo.write('Line({0}) = '.format(nl+1)+r'{'+'{0},{1}'.format(j+2,1)+r'}'+'; \n')
        #
        # geo.write('Curve Loop({0}) = '.format(1)+r'{')
        # #for j in range(0,Npt-1):
        # for j in range(0,Npt-2):
        #     geo.write('{0}, '.format(j+1))
        # geo.write('{0}'.format(j+2)+'}; \n')
        # #geo.write('{0}'.format(j+2)+'}; \n')
        #
        #
        # # geo.write('Extrude {{0,{},0}} {{\n'.format(5))
        # # geo.write('\tLine{{{}}};Layers{{10}};Recombine;\n'.format(nl+2))
        # # geo.write('}\n')
        #
        #
        # # geo.write('Line Loop({0}) = '.format(nl+3)+r'{'+'{0}'.format(nl+2)+r'};'+' \n')
        #
        # geo.write('// Volume number for whole domain.\n')
        # geo.write('Physical Surface({}) = {{{}}};\n'.format(6,5))
        # geo.write('// Top and bottom of the glacier.\n')
        # geo.write('Physical Line({}) = {{{}}};\n'.format(7,1))
        # geo.write('Physical Line({}) = {{{}}};\n'.format(8,2))
        # geo.write('// Left  Wall.\n')
        # geo.write('Physical Line({}) = {{{}}};\n'.format(9,3))
        # geo.write('// Right Wall.\n')
        # geo.write('Physical Line({}) = {{{}}};\n'.format(10,4))


        ########################################################################
        # THIS WORKS .......... kinda
        ########################################################################
        nl=0
        #geo.write('\nBSpline(1) = {')
        geo.write('\nLine(1) = {')
        for j in range(0,Npt-1):
            #if j % 25 == 0 or j == Npt-1:
            nps=nps-1
            geo.write('{}, '.format(nps))

        geo.write('{}}};\n'.format(nps-1))

        # Extrude the mesh
        geo.write('Extrude {{0,{},0}} {{\n'.format(5))
        geo.write('\tLine{1};Layers{10};Recombine;\n')
        geo.write('}\n')

        #geo.write('Physical Surface({}) = {{{}}};\n'.format(2,5))
        geo.write('// Volume number for whole domain.\n')
        geo.write('Physical Surface({}) = {{{}}};\n'.format(6,5))
        geo.write('// Top and bottom of the glacier.\n')
        geo.write('Physical Line({}) = {{{}}};\n'.format(7,1))
        geo.write('Physical Line({}) = {{{}}};\n'.format(8,2))
        geo.write('// Left  Wall.\n')
        geo.write('Physical Line({}) = {{{}}};\n'.format(9,3))
        geo.write('// Right Wall.\n')
        geo.write('Physical Line({}) = {{{}}};\n'.format(10,4))
    geo.close()

# -----------------------
if __name__ == "__main__":
    main(sys.argv[1:])
